@@ -11,14 +11,33 @@ macro_rules! coercible_trait {
         }
 
         unsafe impl $crate::container::InnermostTypeId for dyn $trait {
-            fn innermost_type_id(&self) -> ::core::any::TypeId {
-                ::core::any::Any::type_id(self)
+            #[cfg_attr(feature = "trace", $crate::tracing::instrument(skip_all))]
+            fn innermost_type_id(
+                &self,
+            ) -> Result<::core::any::TypeId, $crate::container::TypeIdDeterminationError> {
+                let type_id = ::core::any::Any::type_id(self);
+                #[cfg(feature = "trace")]
+                $crate::tracing::info!("found type_id {:?}", type_id);
+                Ok(type_id)
             }
         }
     };
 }
 
 macro_rules! coercibles {
+    (
+        <$lt:lifetime, $t:ident, $u:ident>($self:ident, $metadata:ident) {
+            $(#[$feature:literal])? $ty:ty => $coerced:ty $($coerce:block)? as _,
+            $($rest:tt)*
+        }
+    ) => {
+        coercibles! {
+            <$lt, $t, $u>($self, $metadata) {
+                $(#[$feature])? $ty => $coerced $($coerce)? as { (**$self).innermost_type_id() },
+                $($rest)*
+            }
+        }
+    };
     (
         <$lt:lifetime, $t:ident, $u:ident>($self:ident, $metadata:ident) {
             $(#[$feature:literal])? $ty:ty => $coerced:ty $($coerce:block)? as $type:block,
@@ -33,7 +52,10 @@ macro_rules! coercibles {
         where
             $t: ?::core::marker::Sized + $crate::container::InnermostTypeId,
         {
-            fn innermost_type_id(&$self) -> ::core::any::TypeId $type
+            #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+                Self = ::core::any::type_name::<Self>(),
+            )))]
+            fn innermost_type_id(&$self) -> Result<::core::any::TypeId, $crate::container::TypeIdDeterminationError> $type
         }
 
         coercibles! {
@@ -43,19 +65,6 @@ macro_rules! coercibles {
             }
         }
     };
-    (
-        <$lt:lifetime, $t:ident, $u:ident>($self:ident, $metadata:ident) {
-            $(#[$feature:literal])? $ty:ty => $coerced:ty $($coerce:block)? as _,
-            $($rest:tt)*
-        }
-    ) => {
-        coercibles! {
-            <$lt, $t, $u>($self, $metadata) {
-                $(#[$feature])? $ty => $coerced $($coerce)? as { (**$self).innermost_type_id() },
-                $($rest)*
-            }
-        }
-    };
     (
         <$lt:lifetime, $t:ident, $u:ident>($self:ident, $metadata:ident) {
             $(#[$feature:literal])? $ty:ty => $coerced:ty $coerce:block,
@@ -70,6 +79,10 @@ macro_rules! coercibles {
         where
             $t: ?::core::marker::Sized + $crate::container::Coercible,
         {
+            #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+                Self = ::core::any::type_name::<Self>(),
+                U = ::core::any::type_name::<U>(),
+            )))]
             unsafe fn coerce<U>($self, $metadata: $crate::container::Metadata<$crate::container::Coerced<Self::Inner, U>>) -> Self::Coerced<U>
             where
                 U: ?::core::marker::Sized,
