@@ -14,6 +14,9 @@
 mod macros;
 mod impls;
 
+#[cfg(test)]
+mod tests;
+
 use core::{any::TypeId, ptr};
 
 /// The [`Pointee::Metadata`][ptr::Pointee::Metadata] of `U`.
@@ -37,12 +40,16 @@ pub unsafe trait Coercible {
     /// type Coerced<U: 'static + ?Sized>: ?Sized
     /// ```
     ///
-    /// For example: `&mut T` becomes `&mut U`, `Box<T>` becomes `Box<U>`, etc.
-    /// A `dyn Trait` will generally just become `U`.
+    /// For example: `&mut T` becomes `&mut T::Coerced<U>`, `Box<T>` becomes
+    /// `Box<T::Coerced<U>>`, etc.  A `dyn Trait` will generally just become
+    /// `U`.
     type Coerced<U: 'static + ?Sized>: ?Sized;
 
-    /// Basically the same as [`Deref::Target`][core::ops::Deref::Target], but
-    /// without any dereferencing so is safe for raw pointers too.
+    /// The directly contained type, similar to
+    /// [`Deref::Target`][core::ops::Deref::Target] but without any
+    /// dereferencing (so is safe for raw pointers too).  For example: `T` is
+    /// the inner type of an `&mut T`, a `Box<T>`, etc.  The inner type of a
+    /// leaf such as `dyn Trait` is `Self`.
     type Inner: ?Sized + Coercible;
 
     /// The ultimate type whose
@@ -60,6 +67,19 @@ pub unsafe trait Coercible {
 #[allow(type_alias_bounds)]
 pub type Coerced<T: Coercible, U> = T::Coerced<U>;
 
+/// Error that arose whilst determining a pointee's concrete type.
+#[derive(Debug, Eq, PartialEq)]
+#[cfg_attr(feature = "thiserror", derive(thiserror::Error))]
+pub enum TypeIdDeterminationError {
+    /// The concrete type could not be determined because the pointer traverses
+    /// a weak reference to some data that is no longer available.
+    #[cfg_attr(feature = "thiserror", error("unable to upgrade {type_name}"))]
+    UnableToUpgradeWeakReference {
+        /// The name of the Weak reference type that could not be upgraded
+        type_name: &'static str,
+    },
+}
+
 /// A dereferenceable type that inherits
 /// [`Pointee::Metadata`][ptr::Pointee::Metadata] from a contained type,
 /// pointers to which are therefore coercible if that contained type is
@@ -73,9 +93,10 @@ pub unsafe trait InnermostTypeId {
     /// [`<Self as ptr::Pointee>::Metadata`][ptr::Pointee::Metadata].
     ///
     /// Unless `Self` is a leaf, such as a `dyn Trait` (in which case this
-    /// should delegate to a trait method), this should just delegate to the
-    /// contained type's `innermost_type_id`.
-    fn innermost_type_id(&self) -> TypeId;
+    /// should delegate to a trait method like
+    /// [`Any::type_id`][core::any::Any::type_id]), this should just delegate to
+    /// the contained type's `innermost_type_id`.
+    fn innermost_type_id(&self) -> Result<TypeId, TypeIdDeterminationError>;
 }
 
 /// A [`Sized`] type that inherits [`Pointee::Metadata`][ptr::Pointee::Metadata]
