@@ -1,11 +1,24 @@
 //! A database for runtime type information.
 
+pub mod error;
+
+#[cfg(all(test, feature = "std"))]
+mod tests;
+
 #[cfg(any(feature = "std", doc))]
 #[doc(cfg(feature = "std"))]
 pub mod hash_map;
 
 use crate::container::{Coerced, Coercible, InnermostTypeId, Metadata, Pointer};
-use core::{any::TypeId, marker::Unsize, ptr};
+use core::{
+    any::TypeId,
+    marker::{PhantomData, Unsize},
+    ptr,
+};
+use error::{CastError, DatabaseEntryError, DatabaseError};
+
+#[cfg(feature = "trace")]
+use core::any::type_name;
 
 /// A key-value store, where the key is the [`TypeId`] of a concrete Rust type
 /// and the value is that type's [`Metadata<U>`].
@@ -43,6 +56,10 @@ where
     U: ?Sized,
 {
     /// Register concrete type `I` as an implementor of `U`.
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        U = type_name::<U>(),
+        I = type_name::<I>(),
+    )))]
     fn register<I>(&mut self)
     where
         I: 'static + Unsize<U>,
@@ -54,17 +71,35 @@ where
         }
     }
 
-    /// Whether `data` is registered as an implementor of `U`.
-    fn implements<'a, P>(&self, data: P) -> bool
+    /// Attempt to determine the concrete type of the given `data`.
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all))]
+    fn concrete_type_id<'a, P>(&self, data: &P) -> Result<TypeId, DatabaseEntryError<U, P>>
     where
-        P: InnermostTypeId,
+        P: ?Sized + InnermostTypeId,
     {
-        self.contains(data.innermost_type_id())
+        Ok(data.innermost_type_id()?)
+    }
+
+    /// Whether `data` is registered as an implementor of `U`.
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        P = type_name::<P>(),
+        U = type_name::<U>(),
+    )))]
+    fn implements<'a, P>(&self, data: &P) -> Result<bool, DatabaseEntryError<U, P>>
+    where
+        P: ?Sized + InnermostTypeId,
+    {
+        self.concrete_type_id(data)
+            .map(|type_id| self.contains(type_id))
     }
 
     /// Cast `pointer` to `P::Coerced<U>`, if registered as an implementor of
     /// `U`.
-    fn cast<'a, P>(&self, pointer: P) -> Result<P::Coerced<U>, P>
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        P = type_name::<P>(),
+        U = type_name::<U>(),
+    )))]
+    fn cast<'a, P>(&self, pointer: P) -> Result<P::Coerced<U>, CastError<U, P>>
     where
         P: Pointer + InnermostTypeId,
         P::Coerced<U>: Sized,
@@ -72,9 +107,16 @@ where
         Coerced<P::Inner, U>: ptr::Pointee<Metadata = Metadata<U>>,
     {
         unsafe {
-            match self.metadata(pointer.innermost_type_id()) {
-                Some(&metadata) => Ok(pointer.coerce(metadata)),
-                None => Err(pointer),
+            match self.concrete_type_id(&pointer).and_then(|type_id| {
+                self.metadata(type_id).ok_or(
+                    DatabaseEntryError::ConcreteTypeNotRegisteredForTarget {
+                        type_id,
+                        requested_type: PhantomData,
+                    },
+                )
+            }) {
+                Ok(&metadata) => Ok(pointer.coerce(metadata)),
+                Err(source) => Err(CastError { source, pointer }),
             }
         }
     }
@@ -119,3 +161,26 @@ pub unsafe trait TypeDatabase {
     where
         U: 'static + ?Sized;
 }
+
+/// The consumer interface of a `TypeDatabase`.
+pub trait TypeDatabaseExt
+where
+    Self: TypeDatabase,
+{
+    /// Returns a shared/immutable reference to the value of the entry that is
+    /// keyed by `U`.
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        U = type_name::<U>(),
+    )))]
+    fn get_db_entry<U>(&self) -> Result<&Self::Entry<U>, DatabaseError<U>>
+    where
+        U: 'static + ?Sized,
+    {
+        self.get_entry()
+            .ok_or(DatabaseError::RequestedTypeNotInDatabase {
+                requested_type: PhantomData,
+            })
+    }
+}
+
+impl<DB> TypeDatabaseExt for DB where Self: TypeDatabase {}
