@@ -17,8 +17,7 @@
 //! # Example
 //! ```rust
 //! #![feature(generic_associated_types, once_cell)]
-//! # #[cfg(feature = "global")]
-//! # {
+//! # #[cfg(feature = "global")] {
 //!
 //! use rattish::{coercible_trait, rtti_global, GlobalDynCast};
 //! use std::{any::Any, cell::RefCell, fmt, rc::Rc};
@@ -33,13 +32,13 @@
 //! // Casting to an object of trait Bar does not require anything
 //! // special..
 //! trait Bar {
-//!     fn bar(&self) -> i32;
+//!     fn double(&self) -> i32;
 //! }
 //!
 //! struct Qux(i32);
 //! impl Foo for Qux {}
 //! impl Bar for Qux {
-//!     fn bar(&self) -> i32 {
+//!     fn double(&self) -> i32 {
 //!         self.0 * 2
 //!     }
 //! }
@@ -66,16 +65,14 @@
 //!         = foo.dyn_cast::<dyn Bar>().ok().unwrap();
 //!
 //!     // Lo!  We have indeed casted between trait objects.
-//!     let int = bar.borrow().bar();
-//!     assert_eq!(int, 246);
+//!     assert_eq!(bar.borrow().double(), 246);
 //!
 //!     // Enjoy that?  Have another, just for fun:
 //!     let float: &dyn Any = &876.543f32;
-//!     let exp = float.dyn_cast::<dyn fmt::LowerExp>().unwrap();
+//!     let exp = float.dyn_cast::<dyn fmt::LowerExp>().ok().unwrap();
 //!     assert_eq!(format!("{:e}", exp), "8.76543e2");
 //! }
-//! # main()
-//! # }
+//! # main() }
 //! ```
 //!
 //! # Extending rattish to additional pointer/wrapper types
@@ -95,27 +92,39 @@ extern crate alloc;
 pub mod container;
 pub mod db;
 
+#[doc(hidden)]
+#[cfg(feature = "trace")]
+pub use tracing;
+
 use container::{Coerced, Coercible, InnermostTypeId, Metadata, Pointer};
 use core::ptr;
-use db::{TypeDatabase, TypeDatabaseEntryExt};
+use db::{
+    error::{CastError, DatabaseEntryError},
+    TypeDatabaseEntryExt, TypeDatabaseExt,
+};
 
 #[cfg(any(feature = "global", doc))]
 use db::hash_map::DB;
 
+#[cfg(feature = "trace")]
+use core::any::type_name;
+
 /// A type whose implementations can be dynamically determined.
 pub trait DynImplements<'a, DB>
 where
     Self: InnermostTypeId,
-    DB: TypeDatabase,
+    DB: TypeDatabaseExt,
 {
     /// Lookup whether `self`'s ultimate concrete type implements `U` in `db`.
-    fn dyn_implements<U>(&'a self, db: &DB) -> bool
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        Self = type_name::<Self>(),
+        U = type_name::<U>(),
+    )))]
+    fn dyn_implements<U>(&'a self, db: &DB) -> Result<bool, DatabaseEntryError<U, &Self>>
     where
         U: 'static + ?Sized,
     {
-        db.get_entry::<U>()
-            .map(|types| types.implements(self))
-            .unwrap_or(false)
+        db.get_db_entry::<U>()?.implements(&self)
     }
 }
 
@@ -124,19 +133,26 @@ pub trait DynCast<'a, DB>
 where
     Self: Pointer + InnermostTypeId,
     Self::Inner: Coercible,
-    DB: TypeDatabase,
+    DB: TypeDatabaseExt,
 {
     /// Cast `self`'s ultimate concrete type to `U`, if registered as an
     /// implementor of `U` in `db`.
-    fn dyn_cast<U>(self, db: &DB) -> Result<Self::Coerced<U>, Self>
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all, fields(
+        Self = type_name::<Self>(),
+        U = type_name::<U>(),
+    )))]
+    fn dyn_cast<U>(self, db: &DB) -> Result<Self::Coerced<U>, CastError<U, Self>>
     where
         U: 'static + ?Sized,
         Self::Coerced<U>: Sized,
         Coerced<Self::Inner, U>: ptr::Pointee<Metadata = Metadata<U>>,
     {
-        match db.get_entry() {
-            Some(entry) => entry.cast(self),
-            None => Err(self),
+        match db.get_db_entry() {
+            Ok(entry) => entry.cast(self),
+            Err(source) => Err(CastError {
+                source: source.into(),
+                pointer: self,
+            }),
         }
     }
 }
@@ -144,7 +160,7 @@ where
 impl<'a, DB, P: ?Sized> DynImplements<'a, DB> for P
 where
     Self: InnermostTypeId,
-    DB: TypeDatabase,
+    DB: TypeDatabaseExt,
 {
 }
 
@@ -152,7 +168,7 @@ impl<'a, DB, P> DynCast<'a, DB> for P
 where
     Self: Pointer + InnermostTypeId,
     Self::Inner: Coercible,
-    DB: TypeDatabase,
+    DB: TypeDatabaseExt,
 {
 }
 
@@ -166,7 +182,8 @@ where
 {
     /// Lookup whether `self`'s ultimate concrete type implements `U` in the
     /// global [`DB`].
-    fn dyn_implements<U>(&'a self) -> bool
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all))]
+    fn dyn_implements<U>(&'a self) -> Result<bool, DatabaseEntryError<U, &Self>>
     where
         U: 'static + ?Sized,
     {
@@ -184,7 +201,8 @@ where
 {
     /// Cast `self`'s ultimate concrete type to `U`, if registered as an
     /// implementor of `U` in the global [`DB`].
-    fn dyn_cast<U>(self) -> Result<Self::Coerced<U>, Self>
+    #[cfg_attr(feature = "trace", tracing::instrument(skip_all))]
+    fn dyn_cast<U>(self) -> Result<Self::Coerced<U>, CastError<U, Self>>
     where
         U: 'static + ?Sized,
         Self::Coerced<U>: Sized,
